<?php

namespace goodboyalex\php_components_pack\types;

/**
 * Тип-расширение для типа bool.
 *
 * @author Александр Бабаев
 * @package php_components_pack
 * @version 1.0
 * @since 1.1.1
 */
final class BoolEx
{
    /**
     * @var int $False Значение "ложь".
     */
    public const int False = -1;

    /**
     * @var int $Null Значение не установлено (и не истина, и не ложь).
     */
    public const int Null = 0;

    /**
     * @var int $True Значение "истина".
     */
    public const int True = 1;

    /**
     * @var int $Value Хранимое значение.
     */
    private int $Value;

    public function __construct (?bool $value = null)
    {
        // Присваиваем значение
        $this->Value = $value === null ? self::Null : ($value ? self::True : self::False);
    }

    /**
     * Преобразует значение в экземпляр типа.
     *
     * ВАЖНО: при преобразовании в тип будет использована следующая логика:
     *
     * - <code>'true'</code>, <code>'1'</code>, <code>'on'</code>, <code>'yes'</code>, <code>'y'</code>,
     * <code>'t'</code>, <code>true</code>, <code>1</code>, <code>1.0</code> считаются как истина;
     * - <code>'false'</code>, <code>'0'</code>, <code>'off'</code>, <code>'no'</code>, <code>'n'</code>,
     * <code>'f'</code>, <code>false</code>, <code>-1</code>, <code>-1.0</code> считаются как ложь;
     * - все остальные значения считаются как неустановленною (и не истина, и не ложь).
     *
     * @param mixed $value Значение.
     *
     * @return BoolEx Возвращает экземпляр типа.
     */
    public static function Parse (mixed $value): BoolEx
    {
        // Преобразуем значение в тип
        $value = match ($value) {
            'true', '1', 'on', 'yes', 'y', 't', true, 1, 1.0 => self::True,
            'false', '0', 'off', 'no', 'n', 'f', false, -1, -1.0 => self::False,
            default => self::Null
        };

        // Создаем экземпляр типа
        return new BoolEx($value);
    }

    /**
     * Проверяет, является ли значение ложью.
     *
     * @return bool Возвращает, является ли значение ложью.
     */
    public function IsFalse (): bool
    {
        return $this->Value === self::False;
    }

    /**
     * Проверяет, является ли значение истиной.
     *
     * @return bool Возвращает, является ли значение истиной.
     */
    public function IsTrue (): bool
    {
        return $this->Value === self::True;
    }

    /**
     * Возвращает строковое представление значения.
     *
     * @return string Возвращает строковое представление значения.
     */
    public function __toString (): string
    {
        // Преобразуем значение в строку
        return match ($this->Value) {
            self::True => "true",
            self::False => "false",
            self::Null => "null"
        };
    }

    /**
     * Конвертирует значение в bool.
     *
     * @param bool $onNull Значение, возвращаемое при неустановленном значении.
     *
     * @return bool Возвращает значение, конвертированное в bool.
     */
    public function AsBool (bool $onNull = false): bool
    {
        return match ($this->Value) {
            self::True => true,
            self::False => false,
            default => $onNull
        };
    }
}